package com.aftarobot.mobmongo;

import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;

import com.mongodb.client.MongoClient;
import com.mongodb.stitch.android.core.Stitch;
import com.mongodb.stitch.android.core.StitchAppClient;
import com.mongodb.stitch.android.services.mongodb.local.LocalMongoDbService;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * Mobmongo3Plugin
 */
public class Mobmongo3Plugin implements FlutterPlugin, MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel channel;
    private static MongoClient mobileClient;
    private static final String TAG = "MobileMongoPlugin";
    private LocalDBUtil localDBUtil;
    private static final Logger mongoLogger = Logger.getLogger( "com.mongodb" );


    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "mobmongo");
        channel.setMethodCallHandler(this);
        setLogger();
        localDBUtil = new LocalDBUtil();
    }
    public void setLogger() {
        mongoLogger.info("🍎 Mobmongo3Plugin: .... setting MongoDB logging level to 🍎 SEVERE ...");
        mongoLogger.setLevel(Level.SEVERE);
        mongoLogger.info("🍎 Mobmongo3Plugin: MongoDB logging level set to 🍎 SEVERE " +
                "- hopefully no more stupid, useless mongodb logs");
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
        try {
            setupLocalDatabase("localAppX");
            switch (call.method) {
                case "deleteMany":
                    Map<String, Object> map = call.arguments();
                    long resultDelete = localDBUtil.deleteMany(mobileClient, map);
                    result.success(resultDelete);
                    break;
                case "query":
                    Map<String, Object> queryArgs = call.arguments();
                    Object list = localDBUtil.query(mobileClient, queryArgs);
                    result.success(list);
                    break;
                case "createIndex":
                    Map<String, Object> arguments = call.arguments();
                    localDBUtil.createIndex(mobileClient, arguments);
                    result.success("Index created");
                    break;

                case "insert":
                    Map<String, Object> insArgs = call.arguments();
                    String objectId = localDBUtil.insert(mobileClient, insArgs);
                    result.success(objectId);
                    break;
                case "addToArray":
                    Map<String, Object> arrArgs = call.arguments();
                    long arrResult = localDBUtil.addToArray(mobileClient, arrArgs);
                    result.success(arrResult);
                    break;
                case "update":
                    Map<String, Object> replaceArgs = call.arguments();
                    long replaceResult = localDBUtil.update(mobileClient, replaceArgs);
                    result.success(replaceResult);
                    break;
                case "delete":
                    Map<String, Object> deleteArgs = call.arguments();
                    Object deleteResult = localDBUtil.delete(mobileClient, deleteArgs);
                    result.success(deleteResult);
                    break;
                case "getAll":
                    Map<String, Object> getArgs = call.arguments();
                    Object resultList = localDBUtil.getAll(mobileClient, getArgs);
                    result.success(resultList);

                    break;
                case "getOne":
                    Map<String, Object> oneArgs = call.arguments();
                    Object document = localDBUtil.getOne(mobileClient, oneArgs);
                    result.success(document);
                    break;
                case "sync":
                    final Map<String, Object> syncArgs = call.arguments();
                    result.error("014", "\uD83C\uDF36  Sync not appropriate for \uD83C\uDF36 LOCAL \uD83C\uDF36  database", "");
                    break;
                case "getPlatformVersion":
                    result.success("\uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 ANDROID "
                            + android.os.Build.VERSION.RELEASE + " " + Build.DEVICE
                            + " " + Build.MODEL);
                    break;
                case "setAppID":
                    String appID = call.argument("appID");
                    setupLocalDatabase(appID);
                    break;
                default:
                    Log.e(TAG, "onMethodCall: \uD83D\uDC7F \uD83D\uDC7F \uD83D\uDC7F \uD83D\uDC7F  method "
                            + call.method + " is not implemented, Houston, help!");
                    result.notImplemented();
                    break;
            }
        } catch (Exception e) {
            Log.e(TAG, "onMethodCall: ", e);
            result.error("002", e.getMessage(), "onMethodCall: error of some sort. Houston kinda error :( 🍎 🍎 🍎 ");
        }
    }

    private void setupLocalDatabase(String key) {
        if (mobileClient != null) return;
        Log.d(TAG, "\uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38" + " setupLocalDatabase ....Stitch.initializeDefaultAppClient: ");
        StitchAppClient client = Stitch.initializeDefaultAppClient(key);
        mobileClient =
                client.getServiceClient(LocalMongoDbService.clientFactory);

        Log.d(TAG, "\uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38" +
                "  setupLocalDatabase: We cool with LOCAL appID: " +
                "🍎 " + key + " 🍎 on the wild side   \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38");
        for (String n : mobileClient.listDatabaseNames()) {
            Log.d(TAG, "\uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 \uD83C\uDF38 " +
                    "database: " + n  + "🍎 🍎 🍎 🍎 ");
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

}
